# GeoCouch 0.0.1
# Licensed under MIT.
#
# Copyright (c) 2008 Volker Mische (http://vmx.cx/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import os
import sys
import apsw
import simplejson as json
from couchdb.client import Server, ResourceNotFound
import logging

SPATIALTE_MODULE = '/home/vmx/download/spatial/spatialite-2.2/src/versioninfo/.libs/libspatialite.so'
COUCH_SERVER = 'http://localhost:5984/'
GEODB = '/home/vmx/src/couchdb/geo/geocouch.sqlite'
LOG_FILE = '/home/vmx/src/couchdb/geo/geoquery003.log'


def main(args=None):
    logging.basicConfig(
            level=logging.DEBUG,
            filename=LOG_FILE,
            format='%(levelname) 10s %(asctime)s %(message)s')

    log = logging.getLogger('geoquery')

    geodb_name = GEODB
    geodb_conn = apsw.Connection(geodb_name)
    geodb_conn.enableloadextension(True)
    geodb_conn.loadextension(SPATIALTE_MODULE)
    cursor = geodb_conn.cursor()

    couch_server = Server(COUCH_SERVER)

    geocouch_meta = {}
    for db_name in couch_server:
        # database is geoenabled
        if '_design/_geocouch' in couch_server[db_name]:
            geocouch_meta[db_name] = couch_server[db_name]['_design/_geocouch']

    srid = 32632

    while 1:
        rawinput = sys.stdin.readline()

        if rawinput != '':
            log.debug('external query: %s', rawinput)

            data = json.loads(rawinput)

            if data['db'] in geocouch_meta:
                db_name = data['db']

                if 'srid' in data['query']:
                    srid = data['query']['srid']
                else:
                    srid = geocouch_meta[db_name]['srid']


                if 'q' in data['query']:
                    try:
                        query = json.loads(data['query']['q'])
                    except ValueError, e:
                        sys.stdout.write(
                                '{"code": 404, '
                                '''"body": "Couldn't parse JSON: %s"}\n'''
                                % e.message)

                    query = json.loads(data['query']['q'])

                    try:
                        geom_name = query['geom']
                    except KeyError:
                        sys.stdout.write(
                                '{"code": 404, '
                                '"body": "No geometry column specified."}\n')

                    if 'bbox' in query:
                        bbox = query['bbox']

                        result = []
                        geodb_query = '''\
                            SELECT docid, X(%s), Y(%s) FROM %s WHERE
                            MBRContains(BuildMBR(%f, %f, %f, %f, %s), %s);
                            ''' % (geom_name, geom_name, db_name,
                                   bbox[0], bbox[1], bbox[2], bbox[3],
                                   srid, geom_name)

                        try:
                            for docid, x, y in cursor.execute(geodb_query):
                                result.append('%s %f %f' % (docid, x, y))
                        except apsw.SQLError, e:
                            sys.stdout.write('{"code": 404, "body": "%s"}\n'
                                             % e.message)
                            sys.stdout.flush()
                            continue

                        sys.stdout.write('{"code": 200, "body": "%s"}\n'
                                          % '<br>'.join(result))
                        sys.stdout.flush()
                        continue

                    # nothing matched
                    sys.stdout.write('{"code": 404, "body": '
                                     '''"Query %s isn't supported."}\n'''
                                     % query)
            else:
                # nothing matched
                sys.stdout.write('{"code": 404, '
                                 '"body": "Database %s is not geo-enabled."}\n'
                                 % db_name)

            sys.stdout.flush()

        else:
            sys.stdout.flush()
            geodb_conn.close(True)
            self.log.info('exit.')
            break


if __name__ == "__main__":
    sys.exit(main())

